using System;
using System.Collections.Generic;
using System.Text;
using PhoenixDLL.Class.Managers;
using System.Drawing.Printing;
using System.Drawing;
using System.Windows.Forms;
using PhoenixDLL.Class;
using PhoenixDLL.Print.Root;

namespace PhoenixDLL.Print
{
    /// <summary>
    /// Prescriptions  imprimer
    /// </summary>
    public class PrescriptionPrint : RootPrint
    {
        #region Variables
        /// <summary>
        /// Liste des prescriptions
        /// </summary>
        private List<PrescriptionManager> vList = new List<PrescriptionManager>();

        /// <summary>
        /// Header de la prescription
        /// </summary>
        private string vHeader = "Luc LECOQ D.O. \r\nOstopathe M.S.B.O., REO"
            + "\r\nRue des Eburons, 62\r\n4000 LIEGE - Tl. 04/252.95.53";

        /// <summary>
        /// Ville du cabinet
        /// </summary>
        private string vVille = "Lige ";



        #region Impression
        /// <summary>
        /// Bool qui dit si le Header a t dj fait
        /// </summary>
        private bool vHeaderDone = false;
        /// <summary>
        /// Buffer de prescriptions
        /// </summary>
        private string[,] Buffer;
        /// <summary>
        /// Largeur maximum des mdicaments
        /// </summary>
        SizeF MaxWidth;
        #endregion
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public PrescriptionPrint() : base() { }

        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_Prescriptions">Liste des prescriptions  imprimer</param>
        public PrescriptionPrint(List<PrescriptionManager> _Prescriptions)
            : base()
        {
            vList = _Prescriptions;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Fait le traitement d'un vnement d'impression
        /// </summary>
        /// <param name="e"></param>
        protected override void ProcessPrint(PrintPageEventArgs e)
        {
            try
            {
                Font font = new Font("Times New Roman", 12);
                PointF point = new PointF(e.PageSettings.Margins.Left, e.PageSettings.Margins.Top);

                if (!vHeaderDone)
                {
                    #region Variables
                    Buffer = new string[vList.Count, 2];
                    MaxWidth = new SizeF(e.PageSettings.Margins.Left, 0);
                    if (vList.Count < 1) return;
                    #endregion

                    #region Header
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Bold);

                    #region Date
                    string Date = vVille + ", " + vList[0].Date.ToLongDateString();
                    SizeF size = e.Graphics.MeasureString(Date, font);
                    e.Graphics.DrawString(Date, font, new SolidBrush(Color.Black),
                        new PointF(e.PageBounds.Width - (e.PageSettings.Margins.Right + size.Width), point.Y));
                    #endregion

                    #region Header
                    e.Graphics.DrawString(vHeader, font, new SolidBrush(Color.Black), point);
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Regular);

                    point = new PointF(e.PageSettings.Margins.Left, point.Y + font.Height * 4 + 20);
                    e.Graphics.DrawLine(new Pen(Color.Gray), point, new PointF(point.X + e.PageSettings.Margins.Right, point.Y));
                    #endregion

                    #region Pour
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Italic);
                    point = new PointF(point.X, point.Y + font.Height);
                    string Sexe = "madame ";
                    if (vList[0].PatientManager.Sex == Sex.Homme)
                        Sexe = "monsieur ";
                    e.Graphics.DrawString("Pour " + Sexe + " " + vList[0].PatientManager.ToString(), font, new SolidBrush(Color.Black), point);
                    font = new Font(font.Name, font.SizeInPoints, FontStyle.Regular);
                    point = new PointF(point.X, point.Y + (2 * font.Height));
                    e.Graphics.DrawLine(new Pen(Color.Gray), point, new PointF(point.X + e.PageSettings.Margins.Right, point.Y));
                    point = new PointF(point.X, point.Y + font.Height * 2);
                    #endregion
                    #endregion

                    #region Taille des prescriptions
                    //On note les prescription et on regarde la taille de la plus longue afin de mettre toutes les
                    //remarques sur le mme colonne
                    for (int i = 0; i < vList.Count; i++)
                    {
                        Buffer[i, 0] = vList[i].DrugsManager.Name + " (" + vList[i].DrugsManager.DrugsType.Type + ")";
                        Buffer[i, 1] = vList[i].Remark.Replace("\r\n", " "); ;
                        SizeF StringSize = e.Graphics.MeasureString(Buffer[i, 0], font);
                        if (StringSize.Width > MaxWidth.Width)
                            MaxWidth = new SizeF(StringSize.Width + MaxWidth.Width, 0);
                    }
                    #endregion
                }

                #region Prescription
                int index = -1;
                //On dessine les prescriptions et les remarques sur le mme ligne sauf si elle dpasse la feuille
                //for (;index < vList.Count; index++)
                while (++index < (Buffer.Length / 2))
                {
                    SizeF StringSize = e.Graphics.MeasureString(Buffer[index, 1], font);

                    #region Sur une ligne
                    //Si la remarque tient sur la mme ligne: OK
                    Rectangle rec = new Rectangle(e.MarginBounds.Location, e.MarginBounds.Size);
                    if (rec.Contains(new Point((int)(point.X + StringSize.Width + e.MarginBounds.Left),
                        (int)point.Y)))
                    {
                        point = new PointF(point.X, point.Y + font.Height * 2);

                        #region Test si fin de page
                        //Si on est en fin de page, on retourne en dbut de boucle pour continuer l'impression
                        if (point.Y + font.Height + 60 > e.MarginBounds.Height)
                        {


                            string[,] BBuf = new string[(Buffer.Length / 2) - (index), 2];
                            int k = 0;
                            for (int j = index; j < Buffer.Length / 2; j++, k++)
                            {
                                BBuf[k, 0] = Buffer[j, 0];
                                BBuf[k, 1] = Buffer[j, 1];
                            }
                            Buffer = BBuf;
                            index = 0;

                            e.HasMorePages = true;
                            vHeaderDone = true;
                            return;
                        }
                        #endregion

                        #region Nom du mdicament & Remarques

                        font = new Font(font.Name, font.SizeInPoints, FontStyle.Underline);
                        e.Graphics.DrawString(Buffer[index, 0], font, new SolidBrush(Color.Black), point);
                        font = new Font(font.Name, font.SizeInPoints, FontStyle.Regular);

                        e.Graphics.DrawString(Buffer[index, 1], font,
                            new SolidBrush(Color.Black), new PointF(MaxWidth.Width, point.Y));
                        #endregion
                    }
                    #endregion

                    #region Sur plusieur ligne
                    //Sinon, remarque  la ligne
                    else
                    {
                        point = new PointF(point.X, point.Y + (2 * font.Height));
                        int Lines = (int)(StringSize.Width / e.MarginBounds.Width);

                        #region Une remarque est trop grande pour une feuille
                        if (e.MarginBounds.Height < ((e.PageBounds.Height - e.MarginBounds.Height) * 2)
                            + (font.Height * Lines + 1) + 60)
                            throw new OutOfPageException();
                        #endregion

                        #region Test fin de page
                        //Si on est en fin de page, on retourne en dbut de boucle pour continuer l'impression
                        if (point.Y + (font.Height * Lines) + 60 > e.MarginBounds.Height)
                        {
                            string[,] BBuf = new string[(Buffer.Length / 2) - index, 2];
                            int k = 0;
                            for (int j = index; j < Buffer.Length / 2; j++, k++)
                            {
                                BBuf[k, 0] = Buffer[j, 0];
                                BBuf[k, 1] = Buffer[j, 1];
                            }
                            Buffer = BBuf;
                            index = 0;

                            e.HasMorePages = true;
                            vHeaderDone = true;
                            return;
                        }
                        #endregion

                        font = new Font(font.Name, font.SizeInPoints, FontStyle.Underline);
                        e.Graphics.DrawString(Buffer[index, 0], font, new SolidBrush(Color.Black), point);
                        font = new Font(font.Name, font.SizeInPoints, FontStyle.Regular);
                        point = new PointF(point.X, point.Y + (font.Height * 2));

                        e.Graphics.DrawString(Buffer[index, 1], font, new SolidBrush(Color.Black),
                            new RectangleF(point, new SizeF(e.MarginBounds.Width, Lines * font.Height)));

                        point = new PointF(point.X, point.Y + (Lines * font.Height));
                    }
                    #endregion

                }

                #endregion

                #region Signature
                e.HasMorePages = false;
                vHeaderDone = false;
                e.Graphics.DrawString("Signature:", font, new SolidBrush(Color.Black),
                     new PointF(point.X, point.Y + 2 * (font.Height + 30)));
                #endregion
            }
            catch (OutOfPageException ex)
            {
                e.Graphics.Clear(Color.White);
                MessageBox.Show(ex.Message, "Attention", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Retourne ou initialise la liste des prescriptions  imprimer
        /// </summary>
        public List<PrescriptionManager> PrescriptionList { get { return vList; } set { vList = value; } }
        #endregion
    }
}
